#!/bin/sh
#######################################################
#
# plot the group velocity and tfr. 
#
# Martin Schimmel (schimmel@ictja.csic.es)
#######################################################
\rm -f plot.ps .gmtcommands temp.txt

##################
# SET file names:
##################
spec1=ts-pws_spec.txt
grperr=grp.txt
tseries=ts-pws.sac
allm=srs_1-4_max.dat

##################
# TITLE of figure:
##################
stat=`pwd`

sac << END
r $tseries
sc echo &1,dist > tmp
quit
END
dist=` awk '{print $1}' tmp `
tit=" Stations: $stat, Distance: $dist km"

tfilein=temp.txt
sac2asc $tseries $tfilein  c2 nor

############################
# SET the INPUT PARAMETERS:
############################
echo before head -1 $spec1
set -- ` head -1 $spec1 `
echo $1 $2 $3 $4 $5 $6 $7 $8 

############
#data range:
############
x1=$1
x2=$2
dx=$3
y1=$4 
y2=$5 
dy=0.05
t1=$6
t2=$7
#
#####################
# Make color palette:
#####################
gmt makecpt -Cseis -T.0/1/.05 -Z -N -I  -V > cpt.cpt
cp blue.cpt cpt.cpt
#
##########################################
# INCREMENTS AND RANGES OF THE DATA FILES:
##########################################
I1="-I$dx/$dy"
R1="-R$x1/$x2/$y1/$y2"
#
#############
# PLOT RANGE:
#############
# Group velocity window:
#y1=2.5
#y2=5.5
t1=`echo $dist $y2 | awk '{ print $1 / $2 }' `
t2=`echo $dist $y1 | awk '{ print $1 / $2 }' `
# To avoid problems (y1=0) with the log-scale
R1l="-R$x1/$x2/$y1/$y2"
Rt="-R$t1/$t2/-1/1"
R2="-R$x1/$x2/$y1/$y2"
R0="-R0/1/0/1"
#
###############
# XY PLOT SIZE:
###############
J='-JX7.0/4.0'
Jl='-JX7.0l/4.0'
Jt='-JX7.0/2.'
#
#####################
# SET SIZES, FONTS :
#####################
gmt gmtset FONT_LABEL 12
gmt gmtset MAP_TICK_LENGTH_PRIMARY 0.1
gmt gmtset MAP_TICK_LENGTH_SECONDARY 0.1
gmt gmtset MAP_FRAME_PEN  1
#
#################################
# WRITE PS HEADER AND TIME STAMP:
#################################
echo 0 0  | gmt psxy -R1/2/1/2 -P -JX17/26 -Sp -K > plot.ps
#
####################
#  MAKE GRID-files:
####################
echo gridding ...
gmt surface $spec1 -G"$spec1"N.grd $I1 $R1l -V -h1
awk '{print $1,$2,$4}' $spec1 | gmt surface -G"$spec1".grd $I1 $R1l -V -h1
#
#################
# RETRIEVE INFOS:
#################
gmt grdinfo "$spec1"N.grd
gmt grdinfo "$spec1".grd
#
###################
# Plot time series:
###################
#### Figure a)
gmt psxy < $tfilein $Rt $Jt -W1 -O -V -K -P -Y24 \
-Bxf500a1000+l"Time [s]" -Byf24a72   -BSW  >> plot.ps 
echo 0.1  0.1 "a)" | gmt pstext -O -V $R0 $Jt -K -X-1.5 -Y-1.5 -F+f14p,4+jCR >> plot.ps
echo 0  0 "" | gmt pstext -O -V $R0 $Jt -K -X1.5 -Y1.5 >> plot.ps
#
###########################
echo  ... Before grdimage !
# Contour plot:
###########################
# FIGURE b:
gmt grdimage "$spec1".grd -V -P -O -K $R1l $Jl -Y-6.2 -Ccpt.cpt \
-Bxf5a2:"Frequency [Hz]" -Byf.5a1+l"Group Velocity [km/s]"  -BSWne >> plot.ps
awk  '{print $1,$2}' $grperr | gmt psxy -V -P -O -K $R1l $Jl -Sc0.02 -W1,white >> plot.ps
awk  '{print $1,$3}' $grperr | gmt psxy -V -P -O -K $R1l $Jl -S-0.1 -W1,white >> plot.ps
awk  '{print $1,$4}' $grperr | gmt psxy -V -P -O -K $R1l $Jl -S-0.1 -W1,white >> plot.ps
#### plot MAD
awk  '{print $1,$2,$5}' $grperr | gmt psxy -V -P -O -K $R1l $Jl -Ey -S-0.1  -W1 >> plot.ps
echo 0.1  0.1 "b)" | gmt pstext -O -V $R0 $Jt -K -X-1.5 -Y-1.5 -F+f14p,4+jCR >> plot.ps
echo 0  0 | gmt pstext -O -V $R0 $Jt -K -X1.5 -Y1.5 >> plot.ps
#
# FIGURE c:
gmt grdimage "$spec1"N.grd -V -P -O -K $R1l $Jl -X8.5 -Ccpt.cpt \
-Bxf5a2+l"Frequency [Hz]" -Byf.5a1 -BSWne >> plot.ps
awk  '{print $1,$2}' $grperr | gmt psxy -V -P -O -K $R1l $Jl -Sc0.03 -W2 >> plot.ps
awk  '{print $1,$2}' $allm | gmt psxy -V -P -O -K $R1l $Jl -Sp0.03 -W1 >> plot.ps
awk  '{print $1,$3}' $allm | gmt psxy -V -P -O -K $R1l $Jl -Sp0.03 -W1 >> plot.ps
awk  '{print $1,$4}' $allm | gmt psxy -V -P -O -K $R1l $Jl -Sp0.03 -W1 >> plot.ps
awk  '{print $1,$5}' $allm | gmt psxy -V -P -O -K $R1l $Jl -Sp0.03 -W1 >> plot.ps
echo 0.1  0.1 "c)" | gmt pstext -O -V $R0 $Jt -K -X-1.3 -Y-1.5 -F+f14p,4+jCR >> plot.ps
echo 0  0 " " | gmt pstext -O -V $R0 $Jt -K -X1.3 -Y1.5 >> plot.ps
#
#####################
# Plot color palette:
#####################
gmt psscale -Ccpt.cpt -D7.3c/2.8c/2.5c/.15c -O -K -B0.5 >> plot.ps
#
\rm -f tmp.in tmp
#
#########################
##Trailer: ATTENTION  -K
#########################
echo 0 0  | gmt pstext -O -V  -Y3. -X2 $R2 $J >> plot.ps
# 
\rm -f *grd
###############################
# Clean files which you  
# probably do not need any more:
###############################
\rm -f cpt.cpt temp*.txt cpt1.cpt 
#
###gs plot.ps 
