#!/bin/sh
#######################################################
#
# plot the group velocity and tfr. 
#
# Martin Schimmel (schimmel@ictja.csic.es)
#######################################################
\rm -f plot.ps .gmtcommands temp.txt

##################
# SET file names:
##################
spec1=ts-pws_spec.txt
grperr=grp.txt
tseries=ts-pws.sac
tseries2=`ls -1 2010.0*Z.sacposd | head -1 `
tseries3=ts-pws.sac001
# tseries4 is at present not plotted
tseries4=ts-pws.sac002
tseries5=tl.sac
ug_srs=ug_srs.txt
allm=srs_1-4_max.dat

##################
# TITLE of figure:
##################
stat=`pwd`

sac << END
r $tseries
sc echo &1,dist > tmp
quit
END
dist=` awk '{print $1}' tmp `
tit=" Stations: $stat, Distance: $dist km"

tfilein=temp.txt
tfilein2=temp2.txt
tfilein3=temp3.txt
tfilein4=temp4.txt
tfilein5=temp5.txt
sac2asc $tseries $tfilein  c2 nor
sac2asc $tseries2 $tfilein2  c2 nor
sac2asc $tseries3 $tfilein3  c2 nor
sac2asc $tseries4 $tfilein4  c2 nor
sac2asc $tseries5 $tfilein5  c2 nor

############################
# SET the INPUT PARAMETERS:
############################
echo before head -1 $spec1
set -- ` head -1 $spec1 `
echo $1 $2 $3 $4 $5 $6 $7 $8 

############
#data range:
############
x1=$1
x2=$2
dx=$3
y1=$4 
y2=$5 
dy=0.05
t1=$6
t2=$7
#
#####################
# Make color palette:
#####################
gmt makecpt -Cseis -T.0/1/.05 -Z -N -I  -V > cpt.cpt
cp blue.cpt cpt.cpt
#
##########################################
# INCREMENTS AND RANGES OF THE DATA FILES:
##########################################
I1="-I$dx/$dy"
R1="-R$x1/$x2/$y1/$y2"
#
#############
# PLOT RANGE:
#############
# Group velocity window:
#y1=2.5
#y2=5.5
t1=`echo $dist $y2 | awk '{ print $1 / $2 }' `
t2=`echo $dist $y1 | awk '{ print $1 / $2 }' `
# To avoid problems (y1=0) with the log-scale
R1l="-R$x1/$x2/$y1/$y2"
Rt="-R$t1/$t2/-1/1"
R2="-R$x1/$x2/$y1/$y2"
R0="-R0/1/0/1"
#
###############
# XY PLOT SIZE:
###############
J='-JX7.0/4.0'
Jl='-JX7.0l/4.0'
Jt='-JX7.0/2.'
#
#####################
# SET SIZES, FONTS :
#####################
#gmtset FONT_LABEL 12
#gmtset MAP_TICK_LENGTH_PRIMARY 0.1
#gmtset MAP_TICK_LENGTH_SECONDARY 0.1
#gmtset MAP_FRAME_PEN  1
gmt gmtset FONT_LABEL 10
gmt gmtset FONT_ANNOT_PRIMARY 10
gmt gmtset MAP_TICK_LENGTH_PRIMARY 0.1
gmt gmtset MAP_FRAME_PEN  0p
gmt gmtset MAP_FRAME_WIDTH 1p
gmt gmtset MAP_TICK_PEN_PRIMARY 0p
gmt gmtset MAP_TICK_PEN_SECONDARY 0p
#
#################################
# WRITE PS HEADER AND TIME STAMP:
#################################
echo 0 0  | gmt psxy -R1/2/1/2 -P -JX17/26 -Sp -K > plot.ps
#
####################
#  MAKE GRID-files:
####################
echo gridding ...
gmt surface $spec1 -G"$spec1"N.grd $I1 $R1l -V -h1
awk '{print $1,$2,$4}' $spec1 | gmt surface -G"$spec1".grd $I1 $R1l -V -h1
#
#################
# RETRIEVE INFOS:
#################
gmt grdinfo "$spec1"N.grd
gmt grdinfo "$spec1".grd
#
###################
# Plot time series:
###################
#### Figure a)
gmt psxy < $tfilein2 $Rt $Jt -W1 -O -V -K -P -Y24 \
-Bxf500a1000+l"Time [s]" -Byf24a72 -BSW  >> plot.ps 
echo 0.1  0.1 "(a)" | gmt pstext -O -V $R0 $Jt -K -X-1.5 -Y-1.5 -F+f14p,4+jCR >> plot.ps
echo 0  0 "" | gmt pstext -O -V $R0 $Jt -K -X1.5 -Y1.5 >> plot.ps
#
# Figure b)
gmt psxy < $tfilein3  $Rt $Jt -W1,red -O -V -K -P -X8.5 \
-Bxf500a1000+l"Time [s]" -Byf24a72 -BSW  >> plot.ps 
#
gmt psxy < $tfilein5 $Rt $Jt -W1,lightblue -O -V -K -P >> plot.ps
gmt psxy < $tfilein $Rt $Jt -W1 -O -V -K -P >> plot.ps
echo 0.1  0.1 "(b)" | gmt pstext -O -V $R0 $Jt -K -X-1.3 -Y-1.5 -F+f14p,4+jCR >> plot.ps
echo 0  0 ""| gmt pstext -O -V $R0 $Jt -K -X1.3 -Y1.5 >> plot.ps
#
###########################
echo  ... Before grdimage !
# Contour plot:
###########################
# FIGURE c:
gmt grdimage "$spec1".grd -V -P -O -K $R1l $Jl -Y-6.2 -X-8.5 -Ccpt.cpt \
-Bxf5a2 -Byf.5a1+l"Group Velocity [km/s]"  -BSWne >> plot.ps
awk  '{print $1,$2}' $grperr | gmt psxy -V -P -O -K $R1l $Jl -Sc0.02 -W1,white >> plot.ps
awk  '{print $1,$3}' $grperr | gmt psxy -V -P -O -K $R1l $Jl -S-0.1 -W1,white >> plot.ps
awk  '{print $1,$4}' $grperr | gmt psxy -V -P -O -K $R1l $Jl -S-0.1 -W1,white >> plot.ps
#### plot MAD
awk  '{print $1,$2,$5}' $grperr | gmt psxy -V -P -O -K $R1l $Jl -Ey -S-0.1  -W1 >> plot.ps
echo 0.1  0.1 "(c)" | gmt pstext -O -V $R0 $Jt -K -X-1.5 -Y-1. -F+f14p,4+jCR >> plot.ps
echo 0  0 | gmt pstext -O -V $R0 $Jt -K -X1.5 -Y1. >> plot.ps
#
# Inlet:
gmt gmtset FONT_LABEL 9,yellow
gmt gmtset FONT_ANNOT_PRIMARY 9,yellow
gmt gmtset MAP_ANNOT_OFFSET_PRIMARY        1p
gmt gmtset MAP_ANNOT_OFFSET_SECONDARY      1p
gmt gmtset MAP_LABEL_OFFSET      1p
gmt gmtset MAP_FRAME_PEN faint,yellow
gmt gmtset MAP_TICK_PEN_PRIMARY faint,yellow
gmt gmtset MAP_TICK_PEN_SECONDARY faint,yellow   
gmt gmtset MAP_DEFAULT_PEN default,black 
gmt gmtset MAP_GRID_PEN_PRIMARY default,black 
gmt gmtset MAP_GRID_PEN_SECONDARY default,black 
R5l="-R0.015/0.028/3.60/4.10"
J5l='-JX3.0/1.4'
echo 0 0 | gmt psxy -Sp -V -P -O -K $R5l $J5l -Y2.6 -X4 -Bxf0.001a.005 -Byf.05a.2 -BSW >> plot.ps
awk  '{print $1,$2}' $grperr | gmt psxy -V -P -O -K $R5l $J5l -Sc0.02 -W1,white >> plot.ps
awk  '{print $1,$3}' $grperr | gmt psxy -V -P -O -K $R5l $J5l -S-0.1 -W1,white >> plot.ps
awk  '{print $1,$4}' $grperr | gmt psxy -V -P -O -K $R5l $J5l -S-0.1 -W1,white >> plot.ps
awk  '{print $1,$2,$5}' $grperr | gmt psxy -V -P -O -K $R5l $J5l -Ey -S-0.1  -W1 >> plot.ps
awk  '{print $1,$2}' $grperr | gmt psxy -V -P -O -K $R5l $J5l -Sc0.01 -W0,white >> plot.ps
echo 0 0 | gmt pstext -O -V $R0 $Jt -K -Y-2.6 -X-4 >> plot.ps
gmt gmtset FONT_LABEL 10,black
gmt gmtset FONT_ANNOT_PRIMARY 10,black
gmt gmtset MAP_ANNOT_OFFSET_PRIMARY        5p
gmt gmtset MAP_ANNOT_OFFSET_SECONDARY      5p
gmt gmtset MAP_LABEL_OFFSET      8p
gmt gmtset MAP_FRAME_PEN faint,black
gmt gmtset MAP_TICK_PEN_PRIMARY faint,black
gmt gmtset MAP_TICK_PEN_SECONDARY faint,black
#
# FIGURE d:
gmt grdimage "$spec1"N.grd -V -P -O -K $R1l $Jl -X8.5 -Ccpt.cpt \
-Bxf5a2 -Byf.5a1 -BSWne >> plot.ps
awk  '{print $1,$2}' $grperr | gmt psxy -V -P -O -K $R1l $Jl -Sc0.03 -W2 >> plot.ps
awk  '{print $1,$2}' $allm | gmt psxy -V -P -O -K $R1l $Jl -Sp0.03 -W1 >> plot.ps
awk  '{print $1,$3}' $allm | gmt psxy -V -P -O -K $R1l $Jl -Sp0.03 -W1 >> plot.ps
awk  '{print $1,$4}' $allm | gmt psxy -V -P -O -K $R1l $Jl -Sp0.03 -W1 >> plot.ps
awk  '{print $1,$5}' $allm | gmt psxy -V -P -O -K $R1l $Jl -Sp0.03 -W1 >> plot.ps
echo 0.1  0.1 "(d)" | gmt pstext -O -V $R0 $Jt -K -X-1.3 -Y-1. -F+f14p,4+jCR >> plot.ps
echo 0  0 " " | gmt pstext -O -V $R0 $Jt -K -X1.3 -Y1. >> plot.ps
#
#####################
# Plot color palette:
#####################
gmt psscale -Ccpt.cpt -D7.3c/2.8c/2.5c/.15c -O -K -B0.5 >> plot.ps
#
# FIGURE e:
awk  '{print $1,$2}' $grperr | gmt psxy -V -P -O -K $R1l $Jl -W1 -Sp0.04 \
-X-8.5 -Y-5.3 -Bxf5a2+l"Frequency [Hz]" -Byf.5a1+l"Group Velocity [km/s]"  -BSWne >> plot.ps
awk '{if ($7 == 0) print $1,$2,$3}' $ug_srs > tmp.in
gmt psxy tmp.in -V -P -O -K $R1l $Jl -Sc0.04 >> plot.ps
echo 0.1  0.1 "(e)" | gmt pstext -O -V $R0 $Jt -K -X-1.5 -Y-1. -F+f14p,4+jCR >> plot.ps
echo 0  0 " "| gmt pstext -O -V $R0 $Jt -K -X1.5 -Y1. >> plot.ps
#
# FIGURE f:
gmt makecpt -Cseis  -T0/1/.1 -Z -N -I  -V > cpt1.cpt
awk '{if ($7 == 1) print $1,$2,$3}' $ug_srs > tmp.in
gmt psxy tmp.in -V -P -O -K $R1l $Jl -Sc0.06 -Ccpt1.cpt -X8.5  \
-Bxf5a2+l"Frequency [Hz]" -Byf.5a1     -BSWne >> plot.ps
#echo 0.016 4.5 > tmp
#echo 0.027 4.5 >> tmp
#psxy tmp -V -P -O -K $R1l $Jl -W1,blue,40_10:0 >> plot.ps
#rm -f tmp
#echo 0.03  4.47 10 0 4 CL "expected group arrival" | pstext -O -V $R1l $Jl -K >> plot.ps
echo 0.1  0.1  "(f)" | gmt pstext -O -V $R0 $Jt -K -X-1.3 -Y-1. -F+f14p,4+jCR >> plot.ps
echo 0  0 "" | gmt pstext -O -V $R0 $Jt -K -X1.3 -Y1. >> plot.ps
#
\rm -f tmp.in tmp

#####################
# Plot color palette:
#####################
gmt psscale -Ccpt1.cpt -D7.3c/2.8c/2.5c/.15c -O -K -B0.5 >> plot.ps
#
#########################
##Trailer: ATTENTION  -K
#########################
echo 0 0  | gmt pstext -O -V  -Y3. -X2 $R2 $J >> plot.ps
# 
\rm -f *grd
###############################
# Clean files which you  
# probably do not need any more:
###############################
\rm -f cpt.cpt temp*.txt cpt1.cpt 
#
###gs plot.ps 
