       program ts_pws0_ug
c 
c -----------------------------------------------------------------
c ts_pws0_ug == "t"ime-"s"scale"_p"hase-"w"eighted "s"tack for "0" 
c slowness & group velocity "ug" 
c Version: 1.5                                        
c Author: Martin Schimmel (schimmel@ictja.csic.es)
c -----------------------------------------------------------------
c
c Description:
c ============
c This program is based on "tf_pws0.f".  "tf_pws0.f" performs a
c Phase Weighted Stack (PWS, see Schimmel & Paulssen, 1997 for the
c time domain PWS) in the time-frequency domain employing an S-Transform.
c The theory has been published in Schimmel & Gallart (2007).
c There you find also an explanation why & when the S-transform is
c analytic. To go back into the time domain, we use the inverse
c S-transform from Schimmel & Gallart (2005). This inverse S-transform
c is also explained in the appendix of Schimmel & Gallart (2007).  
c Same approach has been used in Schimmel, Stutzmann, Gallart (2011) 
c to process ambient noise data.
c 
c Ventosa et al. (2008) show that the S-transform can be expressed as a 
c Morlet wavelet transform. I.e., one can move from one to the other
c following the mathematical operations derived in Ventosa et al. (2008).
c This strategy has been used and improved to speed up the tf_pws 
c (Ventosa et al. 2016, in prepraration). Thanks to the increased speed 
c resampling strategies can be employed. Here, we include a group 
c velocity extraction approach. The algorithm is operational (e.g. 
c Haned et al., 2016) and is introduced/described in Schimmel et al. (2016).
c 
c Schimmel M., and H. Paulssen, Noise reduction and detection of weak,
c   coherent signals through phase weighted stacks, GJI, 130, 497-505, 
c   doi:10.1111/j.1365-246X.1997.tb05664.x, 1997.
c Schimmel M., and J. Gallart, The inverse S Transform in filters with 
c   time-frequency localization , IEEE Transactions on Signal 
c   Processing, 53 (11), 4417-4422, doi:10.1109/TSP.2005.857065, 2005.
c Schimmel M., and J. Gallart, Frequency-dependent phase coherence 
c   for noise suppression in seismic array data, J. Geophys. Res.,  
c   112, B04303, doi:10.1029/2006JB004680, 2007.
c Schimmel, M., Stutzmann, E., Gallart, J., Using instantaneous 
c   phase coherence for signal extraction from ambient noise data 
c   at a local to a global scale, Geophys. J. Int., 184, 494-506, 
c   doi:10.1111/j.1365-246X.2010.04861.x, 2011.
c Ventosa S., C. Simon, M. Schimmel, J.J. Dañobeitia, and A. Mànuel,
c   The S-transform from a wavelet point of view, IEEE Trans. Signal
c   Proc. 56 (7), doi:10.1109/TSP.2008.917029, 2008.
c Haned, A., Stutzmann, E., Schimmel, M., Kiselev, S., Davaille, A., 
c   Yelles-Chaouche, A., Global tomography using seismic hum, 
c   Geophys. J. Int.,, 204, 1222-1236, doi: 10.1093/gji/ggv516, 2016.
c Ventosa, S., Schimmel, M., Stutzmann, E., in preparation, 2016.
c Schimmel, M., Stutzmann, E., Ventosa, S., Measuring group velocity 
c   in seismic noise correlation studies based on phase coherence and
c   resampling strategies, IEEE Transactions on Geoscience and Remote
c   Sensing, 2016. 
c
c-------------------------------------------------------------------
c
c Last Modifications:
c
c 05/10/2016 : ts_pws0_ugp_1.5.f changed to ts_pws0_ug_1.5.f
c    to have a stable group velocity program.
c    (Phase velocity extraction is still experimental and should
c    be included in next version.)
c
c ==========
c
c I hope this program is useful to you. Please, note I can not take 
c any warranties. I.e., use this program on your own risk. Please, 
c report bugs/improvements/... . Also, if you need any help then do 
c not hesitate to contact me.
c  
c =======================================================================
c
c Dimensions & parameters:
c-------------------------
c nsmax: max # samples in t-series. 
c nfmax: max # wavelets
c nbmax: max # subsets of data. 
c nffmax: max # frequencies.
c nmedx: max # of samples in TFR for determination of medium amplitude. 
      parameter (nsmax=2048,nfmax=24,maxtr=10000000)
      parameter (nbmax=100,nffmax=1024)
c     parameter (nsmax=4096,nfmax=24,maxtr=10000000)
c     parameter (nbmax=100,nffmax=2048)
      parameter (nmedx=5000000)
c if changing dimensions remember to change them in all subroutines.
      character*80 filein
      character*18 par,par1
      character*8 kstnm
      real sig(nsmax),sig2(nsmax),rsig(nsmax)
      real twpi,f1,f2,t1,t2,wu,cycle,dgap,ddgap,drrr
      real(8) dbdum,dbmtr,ramp1,ramp2,ramp3,rphs1,rphs2,rphs3,wdt,dt2
      double complex cts(nsmax,nfmax)
      double complex cts2(nsmax,nfmax),ctsps(nsmax,nfmax)
      double complex cdum
      logical lrm,lkstnm,lgc,lsaco,lallm,lgap
      integer WFid
c SRS:
      real rpro
      integer nbsmp,nseed,iflag,itrbo(nbmax),nugbo
      double complex cts2bo(nbmax,nsmax,nfmax)
      double complex ctspsbo(nbmax,nsmax,nfmax)
      complex ctf2(nsmax,nffmax)
      real rtf2(nsmax,nffmax)
      real rtf2bo(nbmax,nsmax,nffmax)
      real rfmax(nffmax)
      real ugbo(nbmax,nffmax),ugmed(nbmax)
      real grp1,grp2,vg,rmin,dgmax
      real median_fast
      real ugwin,ugrp(nffmax),rmedlim,rmedbo(nmedx)
      real ugdev,ugrpmad(nffmax),augdev(nbmax)
      real dgrpmad12,dgrp12
      logical lfnorm,ldum,lsacbo,ldist,lpwssac,lnomsg,ltfr,lmedlim
      logical lmad,ltest

      twpi=8.0*atan(1.0)
      pi=4.0*atan(1.0)

c DEFAULTS:
C----------
      evla=0.1234
      evlo=0.1234
      stla=0.1122
      stlo=0.1122
      cycle=2.
      lrm=.false.
      lkstnm=.false.
      lgc=.false.
      ldum=.true.
      lsacbo=.false.
      ldist=.false.
      lpwssac=.true.
      lnomsg=.false.
      ltfr=.false.
      lfnorm=.false.
      lmedlim=.false.
      lmad=.false.
      lallm=.false.
      lgap=.false.
      dgap=0.1
      rmedlim=0.
      rmedmin=0.
      nugbo=15
      ugwin=0.01
      wu=2.
      nbsmp=20
      nseed=17
      rpro=0.75
      vg=4.0
      grp1=2.0
      grp2=5.0
      dgmax=1.5
      f1=-2.
      f2=-1.
      rmin=0.5
      dcont=0.05
      ltest=.true.
      dgrp12=3.
      dgrpmad12=3.
      
c INPUTS:
c--------
c  Get number of arguments:
      narg=iargc()
      if (narg.lt.1) then
         write(*,*)' '
         write(*,*)' '
         write(*,*)'TIME-SCALE ZERO LAG PWS & GROUP VELOCITY'//
     &   ' EXTRACTION'
         write(*,*)' '
         write(*,*)' '
         write(*,*)'D I M E N S I O N S   &   D E F A U L T S : '
         write(*,*)'------------------------------------------- '
         write(*,"(A,4I6)")' DIMENSIONS: samples,frequencies'//
     & ',subsets:',nsmax,nffmax,nbmax
         write(*,"(A,2F6.2)")' DEFAULTS: cycle,wu          : ',cycle,wu
         write(*,"(A,4F8.4)")' DEFAULTS: grp1,grp2,vg,rmin : ',
     &     grp1,grp2,vg,rmin
         write(*,"(A,2F8.4,I4)")' DEFAULTS: dgmax,ugwin,nugbo : ',
     &     dgmax,ugwin,nugbo
         write(*,"(A,I4,I5,F8.3)")' DEFAULTS: nbsmp,nseed,rpro  : ',
     &     nbsmp,nseed,rpro
         write(*,*)'DEFAULTS: dcont,medlim      :  ',
     &     dcont,rmedlim
         write(*,*)'DEFAULTS: rm,kstnm,gc,osac  :  ',
     &     lrm,lkstnm,lgc,lsacbo
         write(*,*)'DEFAULTS: dist,pwssac,nomsg :  ',
     &     ldist,lpwssac,lnomsg
         write(*,*)'DEFAULTS: tfro,tfrn,medlim  :  ',ltfr,lfnorm,lmedlim
         write(*,*)'DEFAULTS: mad, gap     :  ',lmad,lgap
         write(*,*)'DEFAULTS: f1=3*vg/dist, f2=min[30*vg/dist,'//
     &     ' 0.7*fnyq]'
         write(*,*)'DEFAULTS: dmad12,dcont12    :  ',dgrpmad12,dgrp12
         call usage
      endif
 
      call getarg (1,filein)

c Get arguments (optional):
c--------------------------
      do iarg=2,narg
        call getarg(iarg,par)
        l=leng(par)
        if (par(1:3).eq.'wu=') then
          par1=par(4:l)
          read(par1,*)wu
        else if (par(1:4).eq.'osac') then
          lsacbo=.true.
        else if (par(1:5).eq.'nomsg') then
          lnomsg=.true. 
        else if (par(1:3).eq.'mad') then
          lmad=.true. 
        else if (par(1:4).eq.'allm') then
          lallm=.true. 
          open(88,file="srs_1-4_max.dat")
          rewind(88)
        else if (par(1:4).eq.'tfro') then
          ltfr=.true. 
        else if (par(1:4).eq.'tfrn') then
          ltfr=.true. 
          lfnorm=.true.
        else if (par(1:8).eq.'nopwssac') then
          lpwssac=.false.
        else if (par(1:2).eq.'gc') then
          lgc=.true.
        else if (par(1:6).eq.'dcont=') then
          par1=par(7:l)
          read(par1,*)dcont
        else if (par(1:8).eq.'dcont12=') then
          par1=par(9:l)
          read(par1,*)dgrp12
        else if (par(1:7).eq.'dmad12=') then
          par1=par(8:l)
          read(par1,*)dgrpmad12
        else if (par(1:7).eq.'medlim=') then
          par1=par(8:l)
          read(par1,*)rmedlim
          lmedlim=.true.
        else if (par(1:6).eq.'nugbo=') then
          par1=par(7:l)
          read(par1,*)rnugbo
          if (rnugbo.gt.1) nugbo=nint(rnugbo)
        else if (par(1:6).eq.'ugwin=') then
          par1=par(7:l)
          read(par1,*)ugwin
        else if (par(1:6).eq.'dgmax=') then
          par1=par(7:l)
          read(par1,*)dgmax
        else if (par(1:5).eq.'rmin=') then
          par1=par(6:l)
          read(par1,*)rmin
        else if (par(1:5).eq.'dist=') then
          par1=par(6:l)
          read(par1,*)rdist
          ldist=.true.
        else if (par(1:6).eq.'kstnm=') then
          kstnm=par(7:l)
          lkstnm=.true.
        else if (par(1:4).eq.'gap=') then
          par1=par(5:l)
          read(par1,*)dgap 
          lgap=.true.
        else if (par(1:3).eq.'gap') then
          lgap=.true.
        else if (par(1:4).eq.'cyc=') then
          par1=par(5:l)
          read(par1,*)cycle
        else if (par(1:3).eq.'f1=') then
          par1=par(4:l)
          read(par1,*)f1
        else if (par(1:3).eq.'f2=') then
          par1=par(4:l)
          read(par1,*)f2
        else if (par(1:3).eq.'vg=') then
          par1=par(4:l)
          read(par1,*)vg
        else if (par(1:5).eq.'grp1=') then
          par1=par(6:l)
          read(par1,*)grp1
        else if (par(1:5).eq.'grp2=') then
          par1=par(6:l)
          read(par1,*)grp2
        else if (par(1:6).eq.'nbsmp=') then
          par1=par(7:l)
          read(par1,*)nbsmp
          if (nbsmp.lt.5) then
            write(*,*)'nbsmp should be larger than 4.'
            stop
          endif
          if (nbsmp.le.1.and.lsaco) lsaco=.false.
          if (nbsmp.gt.nbmax) then
            write(*,*)'nbsmp is > ',nbmax
            write(*,*)'Change dims or decrease nbsmp!'
            stop
          endif
        else if (par(1:5).eq.'seed=') then
          par1=par(6:l)
          read(par1,*)nseed
        else if (par(1:5).eq.'rpro=') then
          par1=par(6:l)
          read(par1,*)rpro 
          if (rpro.lt.0.or.rpro.gt.1.) then
            write(*,*)'USE 0. <= rpro <= 1.'
            stop
          endif 
        else if (par(1:2).eq.'rm') then
          lrm=.true.
        else
          call usage                 
        endif
      enddo

c more parameter checks:
c------------------------
      if (rnugbo.ge.0.and.rnugbo.le.1) nugbo=nint(rnugbo*nbsmp)
      if (nugbo.gt.nbsmp) then
        write(*,*)' WARNING ! nugbo > nbsmp does not make sense.'
        write(*,"(I5,1x,I5)")' nugbo,nbsmp:',nugbo,nbsmp
        stop
      endif
      if (grp2.le.grp1) then
        write(*,*)'grp2 <= grp1 : grp1,grp2',grp1,grp2
        stop
      endif
      if (f2.le.f1) then
        write(*,*)'f2 <= f1 :  f1,f2',f1,f2
        stop
      endif
      dcont=1.0-dcont
      if (rmedlim.eq.0.0) lmedlim=.false.

c Zero arrays:
c-------------
c consumes time and is not needed for most compilers!
      do ns=1,nsmax
        sig(ns)=0.0
        sig2(ns)=0.0
        rsig(ns)=0.0
        do nf=1,nfmax
          cts(ns,nf)=dcmplx(0.0,0.0)
          ctsps(ns,nf)=dcmplx(0.0,0.0)
          cts2(ns,nf)=dcmplx(0.0,0.0)  
          rtf2(ns,nf)=0.0
          do nb=1,nbmax
            ctspsbo(nb,ns,nf)=dcmplx(0.0,0.0)
            cts2bo(nb,ns,nf)=dcmplx(0.0,0.0)
          enddo
        enddo
      enddo
      do nb=1,nbmax
        itrbo(nb)=0
      enddo

c Iniciate random noise generator
c iflag=0 means: use next number in random sequence.
      iflag=0
      call srand(nseed)

c read file with data:
c---------------------
      beg=0.0
      nskip=0
      open(1,file=filein,status="old")
      rewind(1)

c     mxtr=0
c     do while (mxtr.eq.0)
      do itr=1,maxtr
        read(1,*,end=80)filein
        call rsac1(filein,sig,nsamp,beg,dt,nsmax,nerr)
        if (itr.eq.1) then
          beg1=beg
          dt1=dt
          dt2=dt
          if (.not.ldist) then
            call getfhv('stla',stla,nerr)
            call getfhv('stlo',stlo,nerr)
            call getfhv('evla',evla,nerr)
            call getfhv('evlo',evlo,nerr)
            call getfhv("dist",rdist,nerr)
            if (nerr.ne.0) then
             write(*,*)' !!!! PROBLEM !!!!'
             write(*,*)'Program expects a value for distance (either' 
             write(*,*)'through the SAC-header or the program input).'
             stop 
            endif
          endif
          if (.not.lnomsg) write(*,*)'dist:',rdist
          if (lgc) rdist=6371.*twpi - rdist
          if (.not.lnomsg.and.lgc) write(*,*)'use gc dist:',rdist
          if (f1.lt.0.) f1=3.0*vg/rdist
          if (f2.lt.0.) then
             f22=30.0*vg/rdist
             fn=0.7*0.5/dt
             f2=min(f22,fn)
          endif
          t1=rdist/grp2
          t2=rdist/grp1
          if (t1.lt.beg) then
           t1=beg
           grp2=rdist/t1
           if (.not.lnomsg) write(*,*)'grp2 reduced to fit begin time.'
          endif
          tend=beg+(nsamp-1)*dt
          if (t2.gt.tend) then 
           t2=tend 
           grp1=rdist/t2 
           if (.not.lnomsg) write(*,*)'grp1 increased to fit end time.'
          endif
c get power for FFT:
          npow=1
          nsmp=2
 5        if (nsamp.le.nsmp) goto 7
          npow=npow+1
          nsmp=nsmp*2
          goto 5
          stop ' problem with power?'
 7        nsmp2=nsmp/2
          df=1./(dt*float(nsmp))
c on a "sample" base:
          ns1=nint((t1-beg)/dt)+1
          t1=beg+(ns1-1)*dt
          ns2=nint((t2-beg)/dt)+1
          t2=beg+(ns2-1)*dt
          if (abs(t1).lt.dt) t1=dt
          if (abs(t2).lt.dt) t2=dt
          grp1=rdist/t2
          grp2=rdist/t1
          nf1=nint(f1/df)+1
          f1=(nf1-1)*df
          nf2=nint(f2/df)+1
          f2=(nf2-1)*df
        endif

cs wavelet initialization:
cs------------------------
        if (itr.eq.1) then
cs dyadic sampling:
cs        call create_wf (WFid,-1,6,4,nsamp,2.0_8,1.0_8,0,2.0_8*pi,0)
cs uniform sampling:
          call create_wf (WFid,-1,6,4,nsamp,2.0_8,1.0_8,0,2.0_8*pi,1)
        endif

c Some basic checking on the data:
c---------------------------------
        if (nerr.ne.0) stop 'Error in reading sac file'
        if (itr.gt.1) then
           if (nsamp.gt.nsmp) then
              write(*,*)' WARNING: use only ',nsmp,' samples on'//
     & ' trace ',itr
           endif
           if (abs(dt-dt1).gt.dt1*0.01) then
c            stop ' Traces have different dt ! '
             write(*,*)' ATTENTION: trace ',itr,' has a different dt !'
             write(*,*)' ATTENTION: skipping trace ',itr,' .'
             nskip=nskip+1
             goto 999
           endif
           if (abs(beg1-beg).gt.dt1) then
c            stop ' Traces have different begin time.'
             write(*,*)' ATTENTION: trace ',itr,' has a different beg !'
             write(*,*)' ATTENTION: skipping trace ',itr,' .'
             nskip=nskip+1
             goto 999
           endif
        endif

c FALTA NORM

C RM MEAN
c--------
        if (lrm) then
          rmean=0.0
          do i=1,nsamp
            rmean=rmean+sig(i)
          enddo
          do i=1,nsamp
            sig(i)=sig(i)-rmean
          enddo
        endif
c linear t-domain stack
c-----------------------
        do i=1,nsamp
           rsig(i)=sig(i)+rsig(i)
        enddo

cs Wavelet transform:
cs-------------------
cs ST-like
        call fwts(cts,sig,nsamp,nsmax,WFid,1,dt)

        do nf=1,nfmax
           do ns=1,nsamp
              cdum=1000000.*cts(ns,nf)
c             cdum=cts(ns,nf)
              cts2(ns,nf)=cts2(ns,nf)+cdum
              dbdum=zabs(cdum)
              if (dbdum.ge.0.0000001) then
                  ctsps(ns,nf)=ctsps(ns,nf)+cdum/dbdum
             endif
           enddo
        enddo

c Simple Random Sampling:
c trace itr is subjected to an independent Bernoulli trial 
c which determines whether trace itr becomes part of the 
c nb-th sample. Inclusion probablity is same for all traces.
c rpro is the sampling fraction (rpro=n/N).
        do nb=1,nbsmp
           ra=rand(iflag)
           if (ra.le.rpro) then
             itrbo(nb)=itrbo(nb)+1
             do nf=1,nfmax
               do ns=1,nsamp
                 cdum=1000000.*cts(ns,nf)
                 cts2bo(nb,ns,nf)=cts2bo(nb,ns,nf)+cdum
                 dbdum=zabs(cdum)
                 if (dbdum.ge.0.0000001) then
                   ctspsbo(nb,ns,nf)=ctspsbo(nb,ns,nf)+cdum/dbdum
                 endif
               enddo
             enddo
           endif
        enddo
        
 999    continue
        if (.not.lnomsg) then
         if (itr.le.3) write(*,*)' finished with trace #:',itr
         if (itr.eq.4) write(*,*)'           ...'
         if (mod(itr,20).eq.0) then
           write(*,*)' finished with trace #:',itr
           write(*,*)'           ...'
         endif
        endif
c end while loop over all traces!
      enddo
 80   continue
      beg=beg1
      mtr=itr-1-nskip
      if (.not.lnomsg) then
       write(*,*)' total   # traces       :',mtr
       write(*,*)' skipped # traces       :',nskip
       write(*,*)' '
       write(*,*)' rpro, nbsmp            :',rpro, nbsmp
       write(*,*)'# of data set, # of traces'
       do nb=1,nbsmp
         write(*,*)nb,itrbo(nb)
       enddo 
      endif

      dbmtr=dfloat(mtr)
      do nf=1,nfmax
         do ns=1,nsamp
            dbdum = (zabs(ctsps(ns,nf))/dbmtr)**wu
            cts2(ns,nf)=dbdum*cts2(ns,nf)/dbmtr
         enddo
      enddo

      do nb=1,nbsmp
        dbmtr=dfloat(itrbo(nb))
        do nf=1,nfmax
         do ns=1,nsamp
            dbdum = (zabs(ctspsbo(nb,ns,nf))/dbmtr)**wu
            cts2bo(nb,ns,nf)=dbdum*cts2bo(nb,ns,nf)/dbmtr
         enddo
        enddo
      enddo

c INVERSE TRANSFORM:
c-------------------
c     write(*,*)'   ... just before inv transform.'
cs ST-like
      call ifwts(sig2,cts2,nfmax,nsamp,nsmax,WFid,1,dt)

c NEED TO CLEAN NEXT LOOP !!!!!!
c 
c  Handle tf-amplitude spectra, output dispersion curve and error:
c------------------------------------------------------------------
      ns12=ns1+2
      ns22=ns2-2
      do nb=1,nbsmp
        do nf=1,nfmax
          do ns=1,nsamp
            cts2(ns,nf)=cts2bo(nb,ns,nf)
          enddo
        enddo
cs ST-like
        call ifwts(sig,cts2,nfmax,nsamp,nsmax,WFid,1,dt)

c write out the nbsmp t-series (ts_pws):
        if (lsacbo) then
         write(filein,72) 100+nb
 72      format("ts-pws.sac",i3)
         filein(11:11)="0"
         if (lkstnm) then
         call wrsac(filein(1:13),kstnm,beg,dt,sig,nsamp,evla,evlo,
     &    stla,stlo)
         else
         call wrsac(filein(1:13),'nsrs_'//filein(11:13),beg,dt,
     &    sig,nsamp,evla,evlo,stla,stlo)
         endif
        endif

c CLEAN UP: mean of subsets may be not needed
c  S-transform & mean tf-amplitude spectrum
c ------------------------------------------
        call st_4ward(sig,nsamp,cycle,dt,ctf2,nsmp,npow)
        ll=0
        do nff=nf1,nf2
          rfmax(nff)=0.
          do nss=ns1,ns2
            rrr=cabs(ctf2(nss,nff))
            rtf2bo(nb,nss,nff)=rrr
            if (rrr.gt.rfmax(nff)) rfmax(nff)=rrr
            if (lmedlim.and.ll.lt.nmedx) then
              ll=ll+1
              rmedbo(ll)=rrr
            endif 
          enddo
        enddo

        if (lmedlim) then
           rmedmin=rmedlim*median_fast(rmedbo,ll)
        endif

c Find maxima & group velocities:
c--------------------------------
        grr0=-1
        do nff=nf1,nf2
          rrmin=rfmax(nff)*rmin
          rrminx=max(rrmin,rmedmin)
c         write(*,*)rrmin,rmedmin,rrminx
          rmax1=0.0
          rmax2=0.0
          rmax3=0.0
          rmax4=0.0
          nmax1=-1000000
          nmax2=-1000000
          nmax3=-1000000
          nmax4=-1000000
          freq=(nff-1)*df
          do nss=ns12,ns22
            amp=rtf2bo(nb,nss,nff)
            if (amp.gt.rrminx) then
             amp1=rtf2bo(nb,nss-2,nff)
c            amp2=rtf2bo(nb,nss-1,nff)
c            amp3=rtf2bo(nb,nss+1,nff)
             amp4=rtf2bo(nb,nss+2,nff)
             if (amp.gt.rmax1) then
              if (amp1.lt.amp.and.amp4.lt.amp) then
               rmax1=amp
               nmax1=nss
              endif
             else if (amp.gt.rmax2) then
              if (amp1.lt.amp.and.amp4.lt.amp) then
               rmax2=amp
               nmax2=nss
              endif
             else if (amp.gt.rmax3) then
              if (amp1.lt.amp.and.amp4.lt.amp) then
               rmax3=amp
               nmax3=nss
              endif
             else if (amp.gt.rmax4) then
              if (amp1.lt.amp.and.amp4.lt.amp) then
               rmax4=amp
               nmax4=nss
              endif
             endif
            endif
          enddo
c Ug :
c (Tracking Ug even if amplitude below treshhold!)
          tim1=beg+(nmax1-1)*dt
          tim2=beg+(nmax2-1)*dt
          tim3=beg+(nmax3-1)*dt
          tim4=beg+(nmax4-1)*dt
          grr1=rdist/tim1
          grr2=rdist/tim2
          grr3=rdist/tim3
          grr4=rdist/tim4

c write detected maxima:
          if (lallm) write(88,*)freq,grr1,grr2,grr3,grr4

c Finde nun maxima mit kleinstem Abstand zum vorherigem Messwert.
          if (grr0.lt.0.) grr0=grr1
          dg01=abs(grr1-grr0)
          dg02=abs(grr2-grr0)
          dg03=abs(grr3-grr0)
          dg04=abs(grr4-grr0)
          dg00=min(dg01,dg02,dg03,dg04)
          if (dg00.lt.dgmax) then
           if (dg00.eq.abs(grr1-grr0)) then
            grr0=grr1
            if (rmax1.lt.rrminx) grr1=0.0
            ugbo(nb,nff)=grr1
           else if (dg00.eq.abs(grr2-grr0)) then
            grr0=grr2
            if (rmax2.lt.rrminx) grr2=0.0
            ugbo(nb,nff)=grr2
           else if (dg00.eq.abs(grr3-grr0)) then
            grr0=grr3
            if (rmax3.lt.rrminx) grr3=0.0
            ugbo(nb,nff)=grr3
           else if (dg00.eq.abs(grr4-grr0)) then
            grr0=grr4
            if (rmax4.lt.rrminx) grr4=0.0
            ugbo(nb,nff)=grr4
           else
            ugbo(nb,nff)=0.0
           endif
          endif
c END freq loop:
        enddo

c END loop nbsmp:
      enddo

      if (ltest) then
        open(752,file="ug_srs.txt")
        rewind(752)
      endif

c Median ug:
      do nff=nf1,nf2
        ii=0
        do nb=1,nbsmp 
          if (ugbo(nb,nff).gt.0.0) then
           ii=ii+1
           ugmed(ii)=ugbo(nb,nff) 
          endif
        enddo
        if (ii.gt.0) then
          ugmedian=median_fast(ugmed,ii)
          dug=ugmedian*ugwin
          if (lgap) then
           ddug=dug
           do nb=1,nbsmp
             ugdev=abs(ugbo(nb,nff)-ugmedian)
             if (ugdev.gt.ddug) then
               drrr=ddug+dgap
               if (ugdev.lt.drrr) ddug=ugdev
             endif
           enddo
           dug=ddug
          endif
          kk=0
          jj=0
          do nb=1,nbsmp
           ugdev=abs(ugbo(nb,nff)-ugmedian)
           if (ugdev.le.dug) kk=kk+1
           if (ltest) then
             write(752,*)(nff-1)*df,ugbo(nb,nff),kk,
     & ugmedian,ugmedian-ugdev,ugmedian+ugdev,0  
           endif
c Absolut deviations:
           if (lmad.and.ugbo(nb,nff).gt.0.0) then
             jj=jj+1
             augdev(jj)=ugdev
           endif
          enddo
          if (ltest) then
            write(752,*)(nff-1)*df,ugmedian,float(kk)/float(nbsmp),
     & ugmedian,ugmedian-ugdev,ugmedian+ugdev,1
          endif
          if (kk.ge.nugbo) then
             ugrp(nff)=ugmedian
          else
             ugrp(nff)=0.0
          endif
c Median of absolut deviations:
          if (lmad) ugrpmad(nff)=median_fast(augdev,jj)
        endif
      enddo

c S-transform of the 100% tf-PWS:
c--------------------------------
      call st_4ward(sig2,nsamp,cycle,dt,ctf2,nsmp,npow)

      rmaxall=0.
      do nff=nf1,nf2
        rfmax(nff)=0.
        rmp=0.
        do nss=ns1,ns2
          amp=cabs(ctf2(nss,nff))
          rtf2(nss,nff)=amp
          if (amp.gt.rmp) rmp=amp
        enddo
        rfmax(nff)=rmp
        if (rmp.gt.rmaxall) rmaxall=rmp
      enddo

      if (ltfr) then
       open(31,file="ts-pws_spec.txt")
       rewind(31)
c write header:
       write(31,124)f1,f2,df,grp1,grp2,t1,t2,dt
 124   format(2(F18.4,1x),F14.7,1x,F9.4,1x,F9.4,1x,2(F18.4,1x),F14.7)
       do nff=nf1,nf2
         freq=(nff-1)*df
         per=1./freq
c normalize per frequency ?
         if (lfnorm) then
          if (rmaxall.le.0.0) stop 'Problem! amplitudes <= 0 '
          do nss=ns1,ns2
            time=beg+(nss-1)*dt
            grp=rdist/time
            amp=rtf2(nss,nff)/rfmax(nff)
c           write(31,33)freq,grp,amp/rfmax(nff),time,per
            write(31,*)freq,grp,amp,rtf2(nss,nff)/rmaxall,time,per
          enddo
         else
          do nss=ns1,ns2
            time=beg+(nss-1)*dt
            grp=rdist/time
            write(31,*)freq,grp,rtf2(nss,nff)/rmaxall,time,per
          enddo
         endif
       enddo
       close(31)
c33    format(F11.7,F8.4,F15.6,F14.6,F11.5)
      endif

c Work on 100% tf-PWS:
c---------------------
      open(97,file="grp.txt")
      rewind(97)
      nbeg=nint(beg/dt)-1
      twpidt=twpi*dt
      kkff1=1
      do nff=nf1,nf2
        uugg=ugrp(nff)
        if (uugg.gt.0.0) then
         freq=(nff-1)*df

         ldum=.true.
         nsmed=nint(rdist/uugg/dt)-nbeg
         ns=0
         do while (ldum)
          nssm=nsmed+ns
          nssn=nsmed-ns
          ampm1=rtf2(nssm,nff)
          ampm2=rtf2(nssm-1,nff)
          ampm3=rtf2(nssm+1,nff)
          ampn1=rtf2(nssn,nff)
          ampn2=rtf2(nssn-1,nff)
          ampn3=rtf2(nssn+1,nff)
          if (nssm.lt.ns2) then
           if (ampm1.gt.ampm2.and.ampm1.gt.ampm3) then
            ldum=.false.
            nssmax=nssm
           endif
          endif
          if (nssn.gt.ns1) then
           if (ampn1.gt.ampn2.and.ampn1.gt.ampn3) then
            if (.not.ldum.and.ampn1.gt.ampm1) then
              nssmax=nssn
            else if (ldum) then
              nssmax=nssn
              ldum=.false.
            endif
           endif
          endif 
          if (nssn.lt.ns1.and.nssm.gt.ns2) ldum=.false.
          ns=ns+1
c enddo while ldum:
         enddo  
        
         if (.not.lnomsg.and.nssn.lt.ns1.and.nssm.gt.ns2) then
            write(*,*)' Ohoh ... this should not happen. '
            write(*,*)' No max found in tf-PWS of all traces!!'
            write(*,*)' Frequency (Hz):',freq
            goto 70
         endif
         if (ldum) goto 70

         if (dcont.gt.0.0) then
c find duugg(nff) 
           amp0=rtf2(nssmax,nff)
           ampmin=amp0*dcont
           ldum=.true.
           do nss=nssmax,ns2
            r1=rtf2(nss,nff)
            if (r1.lt.ampmin.and.ldum) then 
              ldum=.false.
              nsd1=nss
              goto 81
            endif
           enddo
 81        continue
           ldum=.true.
           do nss=nssmax,ns1,-1
            r1=rtf2(nss,nff)
            if (r1.lt.ampmin.and.ldum) then 
              ldum=.false.
              nsd2=nss
              goto 82
            endif
           enddo
 82        continue
c          ugrp0=rdist/(beg+(nssmax-1)*dt)
           ugrp1=rdist/(beg+(nsd1-1)*dt)
           ugrp2=rdist/(beg+(nsd2-1)*dt)
           ugrp12=ugrp1-ugrp2
           ugrpmad12=2*ugrpmad(nff)
c Determine inst freq, interpolated group time & corresponding
c phase. Remember to correct ST phases!:
           wdt=twpidt*freq
           r1=real(ctf2(nssmax-1,nff))
           r2=aimag(ctf2(nssmax-1,nff))
           rphs1=atan2(r2,r1)+wdt
           ramp1=rtf2(nssmax-1,nff)
           r1=real(ctf2(nssmax,nff))
           r2=aimag(ctf2(nssmax,nff))
           rphs2=atan2(r2,r1)
           ramp2=rtf2(nssmax,nff)
           r1=real(ctf2(nssmax+1,nff))
           r2=aimag(ctf2(nssmax+1,nff))
           rphs3=atan2(r2,r1)-wdt
           ramp3=rtf2(nssmax+1,nff)
           call instftg_eq(ramp1,ramp2,ramp3,
     & rphs1,rphs2,rphs3,wdt,rphs0,finst,tg,dt2)
           ugrp0=rdist/(beg+(nssmax-1.0)*dt+tg)
           if (lmad) then
            if (ugrp12.le.dgrp12.and.ugrpmad12.lt.dgrpmad12) then
             write(97,92)freq,ugrp0,ugrp1,ugrp2,ugrpmad(nff),finst
            endif
           else
            if (ugrp12.le.dgrp12) write(97,98)freq,ugrp0,ugrp1,ugrp2,
     & finst
           endif
         else
           if (lmad) then
             if (ugrpmad12.lt.dgrpmad12) then
               write(97,91)freq,ugrp0,ugrpmad(nff),finst
             endif
           else
             write(97,95)freq,ugrp0,finst
           endif
         endif
 70      continue
c endif uugg.gt.0.0  : 
        endif

c enddo nff:
      enddo
 91   format(2(F8.5,1x),2(F7.4,1x))
 92   format(F8.5,4(1x,F7.4),1x,F8.5)
 95   format(2(F8.5,1x),F7.4)
 98   format(F8.5,3(1x,F7.4),1x,F8.5)
      close(97)

cs WF clean up.
      call destroy_wf (WFid)
      
      rmtr=float(mtr)
      do i=1,nsamp
         rsig(i) = rsig(i)/rmtr
      enddo
      
      if (lpwssac) then
      if (lnomsg) write(*,*)'  ... write sac traces: tl.sac, ts-pws.sac'
      if (lkstnm) then
      call wrsac('tl.sac',kstnm,beg,dt,rsig,nsamp,evla,evlo,stla,stlo)
      call wrsac('ts-pws.sac',kstnm,beg,dt,sig2,nsamp,evla,evlo,
     & stla,stlo)
      else
      call wrsac('tl.sac','t-lin',beg,dt,rsig,nsamp,evla,evlo,stla,stlo)
      call wrsac('ts-pws.sac','ts-pws',beg,dt,sig2,nsamp,evla,evlo,
     & stla,stlo)
      endif
      endif

      end
c--------------------------------------------------------------
      SUBROUTINE usage
      write(*,*)'      '
      write(*,*)'      '
      write(*,*)'U S A G E :'
      write(*,*)'-----------'
      write(*,*)'USAGE:  ts_pws0_ug_1.5 filename parameter_list '
      write(*,*)'      '
      write(*,*)'filename: One file name (SAC data format) per line;'//
     & ' traces must have' 
      write(*,*)'   same header variables b,nsmpl,dt.'//
     & ' Further, dist should be set'
      write(*,*)'   in the SAC header or dist should be provided'//
     & ' through the program'
      write(*,*)'   input. RECOMMENDATION: If amplitudes of t-series'//
     & ' < 10E-06 then'
      write(*,*)'   multiply data with some constant factor to'//
     & ' increase amplitudes.'
      write(*,*)'      '
      write(*,*)'parameter_list: rm cyc= wu= seed= rpro= nbsmp= ...'
      write(*,*)'                 '
      write(*,*)'parameters are optional and can be provided in'//
     & ' arbitrary order'
      write(*,*)'without any blank around =.'
      write(*,*)' '
      write(*,*)'++ GENERAL & PREPROCESSING ISSUES: '
      write(*,*)'rm     : Remove mean from input data. (Default: no rm)'
      write(*,*)'cyc=   : Num. of cycles in 2*std. (Default cyc=2.)'
      write(*,*)'wu=    : PWS power. (Default wu=2.)'
      write(*,*)'kstnm= : Optionally specify sac-header (char*8)'
      write(*,*)'dist=  : Provide distance in km (Default: SAC-hdr).'
      write(*,*)'gc     : Use mayor arc rather than minor arc.'
      write(*,*)
      write(*,*)'++ SCREEN & TFR & WAVEFORM OUTPUT : '
      write(*,*)'nomsg  : Reduce screen output. (Default: no nomsg)'
      write(*,*)'osac   : Write out nbsmp stacks (sac files): '
      write(*,*)'         ts-pws.sac001, ts-pws.sac002, ...'
      write(*,*)'         (Default: no output)'
      write(*,*)'nopwssac:No output of stacks (tl.sac & ts-pws.sac)'//
     & ' using all data.'
      write(*,*)'         (Default: output stacks).'
      write(*,*)'tfro   : Output TFR (PWS all traces). '//
     & 'Output: ' 
      write(*,*)'         tf_pws0_spec.txt (5 columns: freq,grp_vel,'//
     & 'amp,time,period)' 
      write(*,*)'         (Default: no tfro)'
      write(*,*)'tfrn   : Use tfrn instead of tfro for freq '//
     & 'normalized output: ' 
      write(*,*)'         tf_pws0_spec.txt (6 columns: freq,grp_vel,'//
     & 'ampn,amp,time,period)' 
      write(*,*)'         amp is normalized to 1 using extremum.'
      write(*,*)'         ampn is amp normalized as function of freq.'
      write(*,*)
      write(*,*)'++ GROUP VELOCITIES:'
      write(*,*)'grp?=  : Group velocity window in km/s (Default '//
     & 'grp1=2. grp2=5.)' 
      write(*,*)'f1,f2= : Frequency range in Hz to limit output.'
      write(*,*)'         Default f1=3*vg/dist f2=min[32*vg/dist,'//
     & ' 0.7*fnyq] vg=4.0km/s'
      write(*,*)'vg=    : Ref. group velocity (km/s) for f1,f2 '//
     & 'default values.' 
      write(*,*)'         (Default vg=4.0).' 
      write(*,*)'dgmax= : Max group velocity (km/s) jump permitted.'//
     & '(Default dgmax=1.5)'
      write(*,*)'         (Start point is the 1st and largest max'//
     & ' moving from low'
      write(*,*)'         to high frequencies. Choosing a small'//
     & ' dgmax increases'
      write(*,*)'         the dependence on f1.) '
      write(*,*)'rmin=  : Min amplitude a maximum (normalized per'//
     & ' frequency)'
      write(*,*)'         can have to be accepted for the group'//
     & ' velocity'
      write(*,*)'         extraction. (Default rmin=0.5)'
      write(*,*)'medlim=: Use only amplitudes > medlim*median '
      write(*,*)'         (median of all amplitudes in '//
     & 'f1,f2,grp1,grp2 window)'
      write(*,*)'         (Default: medlim=0)'
      write(*,*)'         Use max > max(medlim*median,rmin*rmax(f)).'
      write(*,*)'dcont= : Output group velocities and their'//
     & ' uncertainties'
      write(*,*)'         defined by an dcont % amplitude reduction'
      write(*,*)'         in the PWS of all traces. (Default: '//
     & 'dcont=0.05)'
      write(*,*)'         Note this is only an empirical value '//
     & 'not an error measure.'
      write(*,*)
      write(*,*)'++ STATISTICS FOR GROUP VELOCITIES: '
      write(*,*)'nbsmp= : Number of subsidiary datasets (samples).'// 
     & ' (Default nbsmp=20)'
      write(*,*)'seed=  : Random number seed. (Default seed=17)'
      write(*,*)'rpro=  : Probability for a trace to be included in '//
     & 'a subsidiary'
      write(*,*)'         dataset (subset or sample). rpro is also '//
     & 'called sampling ' 
      write(*,*)'         fraction rpro=n/N (N: total number of '//
     & 'traces, n: number'
      write(*,*)'         of traces in subset). I.e., each trace '//
     & 'is subjected to'
      write(*,*)'         an independent Bernoulli trial which '//
     & 'determines whether it' 
      write(*,*)'         becomes part of a subset or not.  (Default'//
     & ' rpro=0.75)'
      write(*,*)'ugwin= : Median group velocity window [med*(1. '//
     & '+/- ugwin).' 
      write(*,*)'         (Default ugwin=0.01).'
      write(*,*)'gap=   : Makes ugwin data adaptive by increasing '//
     & 'ugwin until ' 
      write(*,*)'         either nbsmp measurements are included or'//
     & ' the difference' 
      write(*,*)'         between group velocity measurements > gap.'//
     & ' Units are km/s.' 
      write(*,*)'         Use just gap (without =) to activate the '//
     & 'default gap=0.1.'
      write(*,*)'nugbo= : Min number of detected group velocities '//
     & 'within window'
      write(*,*)'         grp_med*(1. +/- ugwin).'//  
     & ' grp_med is median velocity of all'
      write(*,*)'         subsets. (Default nugbo=15).'
      write(*,*)'         if 0<=nugbo<=1 then nugbo=nint(nbsmp*nugbo).'
      write(*,*)'mad    : Output group velocities and their'//
     & ' uncertainties'
      write(*,*)'         defined by median absolut deviations '//
     & '(MAD), i.e., the median'
      write(*,*)'         of the absolut deviation from the median.' 
      write(*,*)'         Attention: uses only grpvel measurements '//
     & 'encountered within' 
      write(*,*)'         ugwin window, after medlim & rmin treshhold.'
      write(*,*)'         This can quantify the robustness, i.e., how'//
     & ' the measurements' 
      write(*,*)'         cluster within the ugwin window. It is not '//
     & 'a measurement error.' 
      write(*,*)'NOTE that the main purpose of the statistic is the'//
     & ' assignment of a robust'
      write(*,*)'maximum/ridge in the TFR of the finale PWS.'
      write(*,*)'allm   : Output the 4 largest maxima per frequency '//
     & 'and for each of the'
      write(*,*)'         nbsmp data sets. Output is written to'// 
     & ' file srs_1-4_max.dat'
      write(*,*)'         (freq, grp 1st max, ... grp 4th max),'
      write(*,*)'         without considering selection criteria.'
      write(*,*)'dcont12=: Output only grp_vel measurements if the'//  
     & ' corresponding'
      write(*,*)'          velocity range (if dcont<0) is smaller'//  
     & ' than dcont12'
      write(*,*)'          (units: km/s, default: dcont12=3).'  
      write(*,*)'          dcont12 may help to clean/limit the output.'
      write(*,*)'dmad12=: Output only grp_vel measurements if the'//  
     & ' corresponding'
      write(*,*)'          velocity range (if mad is set) is smaller'//
     & ' than dmad12'
      write(*,*)'          (units: km/s, default: dmad12=3).'
      write(*,*)'          dmad12 may help to clean/limit the output.'
      write(*,*)
      write(*,*)'++ OUTPUTS: (units: freq [Hz], velocities [km/s])'
      write(*,*)' (1) grp.txt: File with group velocities (grpvel).'
      write(*,*)'   if dcont=0: freq,grpvel'
      write(*,*)'   if dcont>0: freq,grpvel,grpvel+dcont,grpvel-dcont'
      write(*,*)'   if dcont=0 & mad: freq, grpvel,MAD'
      write(*,*)'   if dcont>0 & mad: freq, grpvel,grpvel+dcont,'//
     & 'grpvel-dcont,MAD'
      write(*,*)' Note, the dcont-measure is generally not symmetric'//
     & ' around grpvel.'
      write(*,*)'  (2) tf_pws0_spec.txt: File with TFR for contouring.'
      write(*,*)'    if tfro: freq,grpvel,amp,time,period'
      write(*,*)'    if tfrn: freq,grpvel,amp/max(freq),time,period'
      write(*,*)' Note, tfrn stands for freq normalized output.'
      write(*,*)'  (3) Waveform data: sac format.'
      write(*,*)'    if osac: output subset stacks ts-pws.sac001, '//
     & 'ts-pws.sac002, ...'
      write(*,*)'    if nopwssac: do not write tl.sac & ts-pws.sac'
      write(*,*)' Note, on default linear stack and PWS of all data'//
     & ' are written out.'
      write(*,*)
      write(*,*)'FINAL REMARKS: This code is a research algorithm '//
     & '(working version)'
      write(*,*)'and has been distributed in the hope to '//
     & 'be useful for your'
      write(*,*)'research. Please, note I can not take any '//
     & 'warranties, i.e., use'
      write(*,*)'this program on your own risk. If you use '//
     & 'results from this'
      write(*,*)'program in publications then, please, cite '//
     & 'Schimmel et al. (2016)'
      write(*,*)' '
      write(*,*)'NEXT: Some options are experimental'//
     & ' and I need to clean'
      write(*,*)'this program for a more straightforward '//
     & 'usage, to improve'
      write(*,*)'some options, output & documentation, '//
     & 'and to add the'
      write(*,*)'phase velocity extraction.'
      write(*,*)' '
      write(*,*)'AUTHOR: Martin Schimmel, 05/10/2016'
      write(*,*)' '
      write(*,*)'Please, do not hesitate to send bugs, comments'//
     & ', improvements,'
      write(*,*)'nice results, ... to schimmel@ictja.csic.es .'
      write(*,*)' '
      stop
      return
      end
c--------------------------------------------------------------
      SUBROUTINE st_4ward(sig,nsamp,cycle,dt,ctf,nsmp,npow)
c
c INPUT: sig,nsamp,cycle,dt,nsmp,npow
c * sig(nsamp) is real time series with sample interval dt.
c * cycle is the number of periods that equals the 2 std of
c   the Gaussian window.
c * nsmp=2**npow  (==> for FFT)
c * dt is sample interval (time).
c OUTPUT: ctf
c * ctf(nsamp,nsmp2+1) is the complex t-f representation of
c   sig(nsamp).
c   nsamp in ctf is the center sample of 'moving Gauss-window'.
c
c  schimmel@ija.csic.es,  07.05.2004
c  last change 22.08.06
c=================================================================

c     parameter (nsmax=16384,nsctf=16384,nfctf=8193)
c     parameter (nsmax=4096,nsctf=4096,nfctf=2048)
      parameter (nsmax=2048,nsctf=2048,nfctf=1024)
c     parameter (nsmax=2048,nsctf=2048,nfctf=1024)
c     parameter (nsmax=4096,nfmax=2048)
c nfmax=nsmax/2 + 1, nsctf=2**n, nsctf>nsmax
      real sig(nsmax)
      complex csig(nsctf),ctf(nsctf,nfctf)
      complex cdum(nsctf)
      integer npow,nsmp,nsamp
      real pi,fact,cycle

      pi=4.0*atan(1.0)
      fact=cycle/2.
      nsmp2=nsmp/2

      do ns=1,nsamp
        csig(ns)=cmplx(sig(ns),0.0)
      enddo
c to go the safe way:
      do ns=nsamp+1,nsctf
        csig(ns)=cmplx(0.0,0.0)
      enddo
c
c Perform FFT:
c-------------
      rn=1.
      call clogc(npow,csig,rn,dt)

c=======================
c Start frequency loop:
c=======================
      pp=-2.0*pi*pi*fact*fact

c for n=0 (zero frequency nf=1):
c (zero mean t-series assumed!!!!)
c---------------------------------
      do ns=1,nsamp
        ctf(ns,1)=cmplx(0.0,0.0)
      enddo

c and now remaining positive frequencies:
c----------------------------------------
      do nf=2,nsmp2+1
        nf1=nf-1
c zero negative freqs:
        do mf=1,nsmp
          cdum(mf)=cmplx(0.0,0.0)
        enddo
        rpp=pp/float(nf1)/float(nf1)
c       do mf=1,nsmp+1
        do mf=1,nsmp
          rmf=float(mf-1)
          rns=float(nsmp-mf+1)
          argp=rpp*rmf*rmf
          argn=rpp*rns*rns
          if (argp.gt.-25.or.argn.gt.-25) then
c shift Fourier spectrum csig(mf) by freq nf-1 to csig(mf+nf-1):
            mn=mf+nf1
            if (mn.gt.nsmp) mn=mn-nsmp
c multiply with localizing Gaussian (generalized voice Gaussian):
            rr=exp(argp)+exp(argn)
            cdum(mf)=csig(mn)*rr
          endif
        enddo
         
c perform ifft to obtain temporal spectral localizations:
        rn=-1.
        call clogc(npow,cdum,rn,dt)
c store in matrix:
        do ns=1,nsamp
          ctf(ns,nf)=cdum(ns)
        enddo
      enddo

      return
      end
c----------------------------------------------------------------
      SUBROUTINE wrsac(name,kst,b,dt,y,nsmpl,evla,evlo,stla,stlo)
 
      real y(1),b,dt,gcarc
      integer nsmpl
      character*(*) kst,name
 
      call newhdr
      call setnhv('npts',nsmpl,nerr)
      call setfhv('delta',dt,nerr)
      call setkhv('kstnm',kst,nerr)
      call setfhv('evla',evla,nerr)
      call setfhv('evlo',evlo,nerr)
      call setfhv('stla',stla,nerr)
      call setfhv('stlo',stlo,nerr)
 
      e=b+(nsmpl-1)*dt
      o=0.
 
      call setfhv('b',b,nerr)
c     call setihv('iztype','io',nerr)
c
      call wsac0(name,dum,y,nerr)
      if(nerr.ne.0) stop 'Error in writing output file'
 
      return
      end
c--------------------------------------------------------------
      SUBROUTINE CLOGC(n,x,sigm,dt)
 
c--- performs fft on signals with length 2**n and sampling interval
c--- of dt seconds (if in the time domain; notice that dt*df=1/2**n).
c--- the signal is stored in x. it may be complex.
c--- the spectrum is returned in x. it is almost always complex.
c--- a time-to-frequency transform is done with sign=+1. (conform
c--- the convention adopted in aki and richards - the alternative
c--- convention may be obtained by taking complex conjugates after
c--- the call to clogc).
c--- the normalization factor 1./twopi occurs in the frequency-to
c--- time transform (again aki&richards).
c--- normalization is such that physical dimensions are respected.
c--- thus, if the time signal is dimensioned in meters, the
c--- resulting spectral density in x is in meters/hz. for example,
c--- if the time signal is the unit sinc function of width dt, centered
c--- at t=0, the spectral density is dt for all values of the frequency.c
c--- array locations: if x contains the spectrum, it has the spectrum
c--- for positive frequencies in the first 2**n/2+1 elements, such that
c--- x(1) is at 0 hz, x(2) at df hertz, and x(2**n/2+1) at the nyquist,
c--- where df=1./(2**n*dt) and the nyquist is 1./(2*dt) hz.
c--- the second half of x contains the spectrum for negative frequencies
c--- such that x(2**n) is at -df, x(2**n-1) at -2*df hz etcetera.
c--- if x contains the time signal, x(1) is at time 0, x(2)
c--- at time dt etc.
c
      dimension x(1),m(25)
      complex x,wk,hold,q
      if(sigm.ge.0.) then
        sign=1.
      else
        sign=-1.
      endif
      lx=2**n
      do 1 i=1,n
    1 m(i)=2**(n-i)
      do 4 l=1,n
      nblock=2**(l-1)
      lblock=lx/nblock
      lbhalf=lblock/2
      k=0
      do 4 iblock=1,nblock
      fk=k
      flx=lx
      v=sign*6.283185308*fk/flx
      wk=cmplx(cos(v),sin(v))
      istart=lblock*(iblock-1)
      do 2 i=1,lbhalf
      j=istart+i
      jh=j+lbhalf
      q=x(jh)*wk
      x(jh)=x(j)-q
      x(j)=x(j)+q
    2 continue
      do 3 i=2,n
      ii=i
      if(k.lt.m(i)) go to 4
    3 k=k-m(i)
    4 k=k+m(ii)
      k=0
      do 7 j=1,lx
      if(k.lt.j) go to 5
      hold=x(j)
      x(j)=x(k+1)
      x(k+1)=hold
    5 do 6 i=1,n
      ii=i
      if(k.lt.m(i)) go to 7
    6 k=k-m(i)
    7 k=k+m(ii)
      if(sign.gt.0.) go to 9
      flx=flx*dt
      do 8 i=1,lx
    8 x(i)=x(i)/flx
      return
    9 do 10 i=1,lx
   10 x(i)=x(i)*dt
      return
      end 
c----------------------------------------------------------------
      SUBROUTINE instftg_eq(y1,y2,y3,p1,p2,p3,
     & wdt,pe,finst,tg,dt)
c
c Assume evenly spaced x-values & quadratic approximation 
c y(x) = (d2/2)*(x-x2)^2 + d1*(x-x2)+ d0 
c d0, d1, d2 are zeroth, first and second derivative
c
c d0=y2, 
c d1=[(y3-y2)/(x3-x2)]-(d2/2)(x3-x2)=[(y2-y1)/(x2-x1)]+(d2/2)(x2-x1)
c d2=2([(y3-y2)/(x3-x2)]-[(y2-y1)/(x2-x1)])/(x3-x1)
c
c Find location of extremum and interpolate phases.
c Output instant. frequency and group time.
c
c Following
c http://www.ebyte.it/library/codesnippets/P3Interpolation.html
c
c wdt= 2pi*f*dt 
c 
c Here, I assume evenly spaced data with dx=1. 
c x1=-1.d0, x2=0.d0, x3=1.d0
c wdt & dt are used to get finst [Hz] and tg [s] right. 
c I.e., tg is returned in seconds with respect to the center x2 
c and tg=0 means that the group time corresponds exactly to the
c time of the center sample.
c
c schimmel@ictja.csic.es
c----------------------------------------------------------------
       real*8 y1,y2,y3,p1,p2,p3,dt
       real*8 wdt,d1,d2,tgg
       real tg,finst

       twpi=8.0*atan(1.0)
       d1=0.5d0*(y3-y1)
       d2=(y3-y2+y1-y2)

       if (d2.ne.0.0d0) then
         tgg=d1/d2
c        if (dabs(tgg).gt.1.0D0) stop 'PROBLEM with finst!'
       else
         tgg=0.D0
       endif

       pd2=p2-p1
       n=nint((pd2-wdt)/twpi)
       p2=p2-n*twpi
       pd3=p3-p2
       n=nint((pd3-wdt)/twpi)
       p3=p3-n*twpi

c interpolate phase:
      d1=0.5d0*(p3-p1)
      pe=p2-0.5d0*d1*tgg

c interpolate phase differences & determine finst:
      pp=(p2-p1)+(tgg+0.5D0)*(p3-p2-p2+p1)
      finst=abs(pp/dt/twpi)
      tg=-tgg*dt

      if (dabs(tgg).gt.1.0000001D0) stop 'PROBLEM with finst!'

      return
      end
c-------------------------------------------------------------
      integer FUNCTION leng(char)
c     leng is the position of the last non blank character
c     in the character variable char
      character char*(*)
      integer*4 l
      l=len(char)
      do 1 leng=l,1,-1
    1 if (char(leng:leng) .ne. ' ') return
      leng=0
      return
      end   
c=============================================================
      real function median_fast(y,num)
c
c Function to compute median. (schimmel@ictja.csic.es)
c---------------------------------------------------------------
      parameter (memax=5000000)
c     double precision y(1),x(memax),rmed
c     double precision rtest,r1,r2,r1exchge,r2exchge
      real y(1),x(memax),rmed,rmed1
      real rtest,r1,r2,r1exchge,r2exchge
      integer num
      logical li1,li2,leven

      if (num.gt.memax) stop ' Change dims in function median_fast.'

c array copy:
      do i=1,num
         x(i)=y(i)
      enddo

      rmed=0.
      meven=mod(num,2)
      leven=.false.
      if (meven.eq.0) leven=.true.

      mid=nint(num/2.)
 1    n1=1
      n2=num

 5    rtest=x(mid)
 7    li1=.false.
      do i1=n1,mid
        r1=x(i1)
        if (r1.gt.rtest) then
           r1exchge=r1
           i1exchge=i1
           li1=.true.
           goto 11
        endif
      enddo
 11   continue

      li2=.false.
      do i2=n2,mid,-1
        r2=x(i2)
        if (r2.lt.rtest) then
          r2exchge=r2
          i2exchge=i2
          li2=.true.
          goto 12
        endif
      enddo
 12   continue

c exchange values and continue:
      if (li1.and.li2) then
        x(i1exchge)=r2exchge
        x(i2exchge)=r1exchge
        n1=i1exchge+1
        n2=i2exchge-1
        goto 7
      endif

c only i2 reached center:
      if (li1.and..not.li2) then
        x(i1exchge)=x(mid)
        x(mid)=r1exchge
        n1=i1exchge+1
        n2=num
        goto 5
      endif

c only i1 reached center:
      if (li2.and..not.li1) then
        x(i2exchge)=x(mid)
        x(mid)=r2exchge
        n1=1
        n2=i2exchge-1
        goto 5
      endif

c i1 & i2 reached center
      if (.not.li1.and..not.li2) rmed=rtest

      if (leven) then
        if (meven.eq.0) then
          mid=mid+1
          meven=1
          rmed1=rmed
          goto 1
        endif
        rmed=(rmed+rmed1)/2.0
      endif

      median_fast=rmed

      return
      end
